package ca.ntro.ntro_app_fx_impl.executable_spec;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import ca.ntro.app.tasks.SimpleTaskCreator;
import ca.ntro.app.tasks.frontend.BlockingFrontendExecutor;
import ca.ntro.app.tasks.frontend.TypedBlockingFrontendExecutor;
import ca.ntro.core.tasks.Task;
import ca.ntro.ntro_app_fx_impl.tasks.TaskCreator;
import ca.ntro.ntro_app_fx_impl.tasks.TaskDescriptor;
import ca.ntro.ntro_app_fx_impl.tasks.WaitsFor;

public class TaskSpec implements SimpleTaskCreator, TaskCreator, Task {

	private List<String>     waitsFors     = new ArrayList<>();
	private boolean          returnsResult = false;
	private transient Object executor      = null;
	
	private transient Map<String, TaskDescriptor> descriptors = new HashMap<>();
	

	@Override
	public Task getTask() {
		return this;
	}

	@Override
	public TaskCreator waitsFor(String taskId) {
		waitsFors.add(taskId);

		return this;
	}

	@Override
	public TaskCreator waitsFor(TaskDescriptor descriptor) {
		String taskId = descriptor.id();
		
		descriptors.put(taskId, descriptor);
		waitsFors.add(taskId);

		return this;
	}

	@Override
	public SimpleTaskCreator executes(BlockingFrontendExecutor executor) {
		returnsResult = false;
		this.executor = executor;

		return this;
	}

	@Override
	public SimpleTaskCreator executesAndReturnsValue(TypedBlockingFrontendExecutor executor) {
		returnsResult = true;
		this.executor = executor;

		return this;
	}

	@Override
	public void cancel() {
	}

	public void addWaitsFors(WaitsFor taskCreator) {
		for(String waitsForId : waitsFors) {
			
			TaskDescriptor descriptor = descriptors.get(waitsForId);
			
			if(descriptor != null) {

				taskCreator.waitsFor(descriptor);

			}else {
				
				taskCreator.waitsFor(waitsForId);
			}
		}
	}

	public void addExecutor(SimpleTaskCreator taskCreator) {
		if(returnsResult && executor != null) {
			
			taskCreator.executesAndReturnsValue((TypedBlockingFrontendExecutor) executor);
			
		}else if(!returnsResult && executor != null) {
			
			taskCreator.executes((BlockingFrontendExecutor) executor);
			
		}
	}

}
